<?php
/**
 * YJShopping MediaWiki extension  version 1.2.4
 *	for details please see: https://kimagurenote.net/kn/YJShopping
 *
 * Copyright (c) 2019-2020 Kimagurenote https://kimagurenote.net/
 * License: Revised BSD license http://opensource.org/licenses/BSD-3-Clause
 *
 * Function:
 *	Mediawiki extension that make item links for several shopping site.
 *
 * Usage:
 *	<yjshopping>free keyword</shopping> on your wiki page.
 *	for details please see: https://kimagurenote.net/kn/YJShopping
 *
 * Dependency:
 *	Yahoo!JAPAN Shopping search API https://developer.yahoo.co.jp/webapi/shopping/shopping/v1/itemsearch.html
 *	PHP 5.4.0+ (using http_build_query() with PHP_QUERY_RFC3986 option)
 *
 * History:
 * 2021.10.15 Version 1.2.4
 *	querytype="book" パラメータを追加、ebookjapanに対応
 * 2021.01.23 Version 1.2.3
 *	A8.netの処理を追加、「NTT-X Store」と「ひかりTVショッピング」に対応
 * 2021.01.19 Version 1.2.2
 *	au PAY マーケット（旧Wowma!）をLinkShareに切り替え
 *	Qoo10（VC、ISBN/JAN非表示）に対応
 *	LinkShare の 'bids' を省略可にした
* 2020.06.04 Version 1.2.1
 *	PHP 7.4 にしたら warning や notice が出るようになったので対策
 * 2020.05.12 Version 1.2.0
 *	商品検索API v3 に対応（以降、API v1 には非対応）
 *	API v3 でISBN検索をすると "Bad Request" が返るため、暫定的にJANコード検索で対応
 * 2020.02.08 Version 1.1.4
 *	ビックカメラ (LinkShare) に対応、キーワードにUTF-8不可（SJIS化＋urlencodeが必要）
 *	target="" の指定に対応（規定は _blank、無効にする場合は $wgYJShoppingTarget に空文字列を指定する）
 * 2019.12.18 Version 1.1.3-3
 *	PayPayモール (ValueCommerce) に対応
 * 2019.07.10 Version 1.1.3-2
 *	ナチュラム (LinkShare) に対応
 * 2019.07.06 Version 1.1.3
 *	イトーヨーカドーネット通販 (ValueCommerce) に対応
 *	display:table の解除
 *	urlencode() を rawurlencode() に変更
 *	API呼出URLの生成に http_build_query() を使う処理に変更
 *	http_build_query() にて PHP_QUERY_RFC3986 を使うため PHP 5.4.0 以降必須
 *	JAN指定時に Sofmap.com, Sofmapリコレ（中古）を出ないようにした
 * 2019.06.02 Version 1.1.2
 *	Sofmap.com, Sofmapリコレ（中古） (LinkShare) に対応
 *	楽天市場 (LinkShare) に対応
 * 2019.05.30 Version 1.1.1
 *	関数出力が後々（MediaWiki側で）勝手にescapeされることがある問題に仮対策（未解決、動作はしているようだが…）
 *	Amazon.co.jp に表記変更、Amazon.co.jp の ValueCommerce MyLink に対応
 *	内部変数 $wgYJShoppingImgStyle を廃止、float:left の直接指定を削除（要CSS）
 *	定数YJSHOPPING_DEBUGを追加、debug code を存置するとともに通常は通さないように
 *	Web Services by Yahoo! JAPAN クレジットを追加 https://developer.yahoo.co.jp/attribution/
 *	バージョン番号表記をMediaWiki似に変更
 * 2019.05.26 Version 1.10
 *	ValueCommerce系の処理を統合して関数に分離、Amazonと楽天の処理も関数に分離
 *	'yj' を 'yahoo' に変更（当面の間、互換性処理を追加）
 *	帯域節約のため、先頭の1件のみ取得するようにした
 *	与えられた引数を mb_convert_encoding() に通すようにした（EUC等を使っているWiki対策）
 * 2019.05.24 Version 1.00
 *	1st test （商品検索API v1 に対応）
 *
 * @file
 * @ingroup Extensions
 * @author Kimagurenote
 * @copyright © 2019 Kimagurenote
 * @license The BSD 3-Clause License
 */

if( !defined( 'MEDIAWIKI' ) ) {
	echo( "This file is an extension to the MediaWiki software and cannot be used standalone.\n" );
	die( 1 );
}

$wgExtensionCredits['parserhook'][] = array(
	'path'           => __FILE__,
	'name'           => 'YJShopping',
	'author'         => array(
		'[https://kimagurenote.net/ Kimagurenote]'
	),
	'url'            => 'https://kimagurenote.net/kn/YJShopping',
	'version'        => '1.2.4',
	'license-name'   => 'BSD-3-Clause',
	'descriptionmsg' => 'yjshopping-desc'
);

/* Internationalization */
$wgMessagesDirs['YJShopping'] = __DIR__ . '/i18n';

/* default values */
#define( 'YJSHOPPING_DEBUG', 'yjshopping' );	/* for debug */

define( 'YJSHOPPING_HOOKID', 'yjshopping' );
define( 'YJSHOPPING_CLASS_DIV', 'yjshopping' );

define( 'YJSHOPPING_CREDIT_URL', 'https://s.yimg.jp/images/yjdn/common/yjdn_attbtn1_125_17.gif' );
define( 'YJSHOPPING_CREDIT_SIZE_X', 125 );
define( 'YJSHOPPING_CREDIT_SIZE_Y', 17 );
define( 'YJSHOPPING_CREDIT_ALT', 'Web Services by Yahoo! JAPAN' );
define( 'YJSHOPPING_CREDIT_TARGET', 'https://developer.yahoo.co.jp/about' );

/* query type identifier */
define( 'YJSHOPPING_QUERYTYPE_ISBN', 'isbn' );
define( 'YJSHOPPING_QUERYTYPE_JAN', 'jan' );
define( 'YJSHOPPING_QUERYTYPE_BOOK', 'book' );

/* operation parameter */
define( 'YJSHOPPING_NAMEMAXLEN', -1 );
define( 'YJSHOPPING_STOREMAXLEN', -1 );
define( 'YJSHOPPING_TRIMMARKER', '…' );

/* Yahoo!JAPAN Shopping API URL (json)
 * https://developer.yahoo.co.jp/webapi/shopping/shopping/v3/itemsearch.html
 */
$wgYJShoppingAPIURL = 'https://shopping.yahooapis.jp/ShoppingWebService/V3/itemSearch';

/* target */
$wgYJShoppingTarget = "_blank";

/* tag classes */
$wgYJShoppingClass['div'] = '';
$wgYJShoppingClass['img'] = 'yjshoppingimg';
$wgYJShoppingClass['name'] = 'yjshoppingname';
$wgYJShoppingClass['store'] = 'yjshoppingstore';
$wgYJShoppingClass['plus'] = 'yjshoppingplus';
$wgYJShoppingClass['yahoo'] = 'shoplinkyahoo';
$wgYJShoppingClass['paypaymall'] = 'shoplinkpaypaymall';
$wgYJShoppingClass['amazon'] = 'shoplinkamazon';
$wgYJShoppingClass['rakuten'] = 'shoplinkrakuten';
$wgYJShoppingClass['biccamera'] = 'shoplinkbiccamera';
$wgYJShoppingClass['sofmap'] = 'shoplinksofmap';
$wgYJShoppingClass['sofmapused'] = 'shoplinksofmapused';
$wgYJShoppingClass['naturum'] = 'shoplinknaturum';
$wgYJShoppingClass['honto'] = 'shoplinkhonto';
$wgYJShoppingClass['ebookjapan'] = 'shoplinkebookjapan';
$wgYJShoppingClass['7net'] = 'shoplink7net';
$wgYJShoppingClass['iyec'] = 'shoplinkiyec';
$wgYJShoppingClass['wowma'] = 'shoplinkwowma';
$wgYJShoppingClass['nttxstore'] = 'shoplinknttxstore';
$wgYJShoppingClass['hikaritv'] = 'shoplinkhikaritv';
$wgYJShoppingClass['qoo10'] = 'shoplinkqoo10';
$wgYJShoppingClass['kakakucom'] = 'shoplinkkakakucom';

/* backward compatibility from API v1 to v3 */
$wgYJShoppingChangeParams['category_id'] = 'genre_category_id';
$wgYJShoppingChangeParams['store_id'] = 'seller_id';
$wgYJShoppingChangeParams['offset'] = 'start';

/* accepted parameters for pass to API */
$wgYJShoppingAcceptParams = array( 'image_size', 'genre_category_id', 'brand_id', 'seller_id', 'start', 'shipping', 'payment', 'delivery_area', 'delivery_day', 'delivery_deadline', 'sort', 'condition' );

/* MediaWiki set hook */
$wgHooks['ParserFirstCallInit'][] = 'wfYJShoppingSetHook';

function wfYJShoppingSetHook( $parser ) {
	$parser->setHook( YJSHOPPING_HOOKID, 'wfRenderYJShopping' );
	return true;
}

/* Main process - The callback function for embed shopping link
 *
 */
function wfRenderYJShopping( $value, $argv, $parser ) {
	global $wgYJShoppingAPIURL, $wgYJShoppingAppID, $wgYJShoppingVCID, $wgYJShoppingChangeParams, $wgYJShoppingAcceptParams, $wgYJShoppingClass, $wgYJShoppingMaxLen, $wgYJShoppingPlus, $wgYJShoppingTarget;

	/* for debug */	defined( 'YJSHOPPING_DEBUG' ) && ( $parser->disableCache() );

	/* search keyword */
	$value = mb_convert_encoding(trim($value), 'UTF8', 'ASCII,JIS,UTF-8,EUC-JP,SJIS-WIN');

	/* require apiurl, appid and keyword(value) */
	if( empty($wgYJShoppingAPIURL) || empty($wgYJShoppingAppID) || empty($value) ) {
		/* for debug */	defined( 'YJSHOPPING_DEBUG' ) && ( wfDebugLog( YJSHOPPING_DEBUG, "wfRenderYJShopping(): required parameter(s) empty: $wgYJShoppingAPIURL, $wgYJShoppingAppID, $value.\n" ) );
		return '';
	}

	/* backward compatibility */
	if( !isset($wgYJShoppingPlus['yahoo']) && !empty($wgYJShoppingPlus['yj']) ) {
		$wgYJShoppingPlus['yahoo'] = $wgYJShoppingPlus['yj'];
		unset($wgYJShoppingPlus['yj']);
	}

	/* generate API URL */
	$apiparam['appid'] = $wgYJShoppingAppID;

	/* ValueCommerce affiliate ID (option)
	 * https://developer.yahoo.co.jp/appendix/shopping/affiliate.html
	 */
	if( !empty($wgYJShoppingVCID) ) {
		$apiparam['affiliate_type'] = 'vc';
		$apiparam['affiliate_id'] = $wgYJShoppingVCID;
	} elseif ( !empty($wgYJShoppingPlus['yahoo']['sid']) && !empty($wgYJShoppingPlus['yahoo']['pid']) ) {
		$apiparam['affiliate_type'] = 'vc';
		$apiparam['affiliate_id'] = 'https://ck.jp.ap.valuecommerce.com/servlet/referral?sid=' . $wgYJShoppingPlus['yahoo']['sid'] . '&pid=' . $wgYJShoppingPlus['yahoo']['pid'] . '&vc_url=';
	}

	/* select query type (default: keyword search) */
	$querytype = @$argv['querytype'];
	if ( strcasecmp($querytype, YJSHOPPING_QUERYTYPE_ISBN) == 0 ) {
		$querytype = YJSHOPPING_QUERYTYPE_ISBN;
		$value = str_replace('-', '', $value);
//		$apiparam['isbn'] = $value;
		$apiparam['jan_code'] = $value;	/* API v3 の ISBN検索の不具合対策 */
	} elseif ( strcasecmp($querytype, YJSHOPPING_QUERYTYPE_JAN) == 0 ) {
		$querytype = YJSHOPPING_QUERYTYPE_JAN;
		$apiparam['jan_code'] = $value;
	} elseif ( strcasecmp($querytype, YJSHOPPING_QUERYTYPE_BOOK) == 0 ) {
		$querytype = YJSHOPPING_QUERYTYPE_BOOK;
		$apiparam['query'] = $value;
	} else {
		$querytype = '';
		$apiparam['query'] = $value;
	}

	/* set default value */
	$apiparam['start'] = '1';

	/* more parameters */
	if( !empty($args = @$argv) ) {
		/* backward compatibility from API v1 */
		foreach( $wgYJShoppingChangeParams as $k => $v ) {
			if( !empty($args[$k]) ) {
				$apiparam[$v] = $args[$k];
				unset($args[$k]);
			}
		}

		/* more parameters if accepted (option) */
		foreach( $wgYJShoppingAcceptParams as $v ) {
			if( !empty($args[$v]) ) {
				$apiparam[$v] = $args[$v];
				unset($args[$v]);
			}
		}
	}

	/* set result count */
	$apiparam['results'] = '1';

	/* generate API URL */
	if( empty( $apiurl = http_build_query($apiparam, '', '&', PHP_QUERY_RFC3986) ) ) {
		/* for debug */	defined( 'YJSHOPPING_DEBUG' ) && ( wfDebugLog( YJSHOPPING_DEBUG, "wfRenderYJShopping(): can't generate api url.\n" ) );
		return '';
	}
	$apiurl = $wgYJShoppingAPIURL . '?' . $apiurl;

	/* call API */
	if( ($retapi = file_get_contents($apiurl)) === false ) {
		/* for debug */	defined( 'YJSHOPPING_DEBUG' ) && ( wfDebugLog( YJSHOPPING_DEBUG, "wfRenderYJShopping(): can't call api $apiurl.\n" ) );
		return '';
	}

	/* for debug */	defined( 'YJSHOPPING_DEBUG' ) && ( wfDebugLog( YJSHOPPING_DEBUG, "wfRenderYJShopping(): API call success.\n$apiurl\n" ) );

	/* decode JSON */
	if( is_null( $resapi = json_decode( mb_convert_encoding( $retapi, 'UTF8', 'ASCII,JIS,UTF-8,EUC-JP,SJIS-WIN'), true ) ) ) {
		/* for debug */	defined( 'YJSHOPPING_DEBUG' ) && ( wfDebugLog( YJSHOPPING_DEBUG, "wfRenderYJShopping(): can't decode json.\n" . $resapi . "\n" ) );
		return '';
	}

	/* no result */
	if( $resapi['totalResultsReturned'] < 1 ) {
		/* for debug */	defined( 'YJSHOPPING_DEBUG' ) && ( wfDebugLog( YJSHOPPING_DEBUG, "wfRenderYJShopping(): no result.\n" . var_export($resapi, TRUE) . "\n" ) );
		return '';
	}

	/* got result, render start */

	/* generate embed HTML tags */
	$ret = '<div class="' . YJSHOPPING_CLASS_DIV;
	if( !empty($wgYJShoppingClass['div']) ) {
		$ret .= ' ' . $wgYJShoppingClass['div'];
	}
	$ret .= '" style="position: relative;">';

	/* image */
	$ret .= '<a href="' . $resapi['hits'][0]['url'] . '"' . ($wgYJShoppingTarget ? ' target="' . $wgYJShoppingTarget . '"' : '') . ' rel="nofollow">';
	$ret .= '<img src="' . $resapi['hits'][0]['image']['medium'];
	if( !empty($wgYJShoppingClass['img']) ) {
		$ret .= '" class="' . $wgYJShoppingClass['img'];
	}
	$ret .= '" style="margin-bottom: 15px;" /></a>';

	/* item name and link */
	$name = $resapi['hits'][0]['name'];
	$maxlen = empty($wgYJShoppingMaxLen['name']) ? YJSHOPPING_NAMEMAXLEN : $wgYJShoppingMaxLen['name'];
	$ret .= '<p';
	if( !empty($wgYJShoppingClass['name']) ) {
		$ret .= ' class="' . $wgYJShoppingClass['name'] . '"';
	}
	$ret .= '><a href="' . $resapi['hits'][0]['url'] . '"' . ($wgYJShoppingTarget ? ' target="' . $wgYJShoppingTarget . '"' : '') . ' rel="nofollow">';
	if( $maxlen < 0 || strlen($name) <= $maxlen ) {
		$ret .= htmlspecialchars($name);
	} else {
		$ret .= htmlspecialchars(mb_strimwidth($name, 0, $maxlen, YJSHOPPING_TRIMMARKER));
	}
	$ret .= '</a></p>';

	/* seller name */
	$text = $resapi['hits'][0]['seller']['name'];
	$maxlen = empty($wgYJShoppingMaxLen['store']) ? YJSHOPPING_STOREMAXLEN : $wgYJShoppingMaxLen['store'];
	$ret .= '<p';
	if( !empty($wgYJShoppingClass['store']) ) {
		$ret .= ' class="' . $wgYJShoppingClass['store'] . '"';
	}
	$ret .= '>';
	if( $maxlen < 0 || strlen($text) <= $maxlen ) {
		$ret .= htmlspecialchars($text);
	} else {
		$ret .= htmlspecialchars(mb_strimwidth($text, 0, $maxlen, YJSHOPPING_TRIMMARKER));
	}
	$ret .= '</p>';

	/* plus links */
	if( !empty($wgYJShoppingPlus) ) {
		$ret .= '<ul';
		if( !empty($wgYJShoppingClass['plus']) ) {
			$ret .= ' class="' . $wgYJShoppingClass['plus'] . '"';
		}
		$ret .= '>';

		/* add Plus links (option) */
		foreach(array_keys($wgYJShoppingPlus) as $k) {
			$retplus = '';
			if( empty($wgYJShoppingPlus[$k]) ) {
				continue;
			} elseif( $k === 'amazon' && gettype($wgYJShoppingPlus[$k]) == 'string' ) {
				/* plus Amazon Associate (option) */
				$retplus = wfRenderYJShoppingPlusAmazon( $wgYJShoppingPlus[$k], $value );
			} elseif( $k === 'rakuten' && !empty($wgYJShoppingPlus[$k]['lsid']) && gettype($wgYJShoppingPlus[$k]['lsid']) == 'string' ) {
				/* plus Rakuten (LinkShare) (option) */
				$retplus = wfRenderYJShoppingPlusRakutenLS( $wgYJShoppingPlus[$k]['lsid'], $value );
			} elseif( $k === 'rakuten' && gettype($wgYJShoppingPlus[$k]) == 'string' ) {
				/* plus Rakuten (option) */
				$retplus = wfRenderYJShoppingPlusRakuten( $wgYJShoppingPlus[$k], $value );
			} elseif( !empty($wgYJShoppingPlus[$k]['sid']) && !empty($wgYJShoppingPlus[$k]['pid']) ) {
				/* plus ValueCommerce (option) */
				if( ($k === 'ebookjapan') && (strcasecmp($querytype, YJSHOPPING_QUERYTYPE_BOOK) != 0) ) {
					/* ebookjapan is book only */
					continue;
				}
				if( ($k === 'honto' || $k === '7net') && empty($querytype) ) {
					/* honto and 7net options are ISBN/JAN or book only */
					continue;
				}
				if( ($k === 'iyec' || $k === 'qoo10') && !empty($querytype) ) {
					/* iyec and qoo10 options are keyword only */
					continue;
				}
				$retplus = wfRenderYJShoppingPlusVC( $k, $wgYJShoppingPlus[$k]['sid'], $wgYJShoppingPlus[$k]['pid'], $value );
			} elseif( !empty($wgYJShoppingPlus[$k]['lsid']) ) {
				/* plus LinkShare (option) */
				if( ($k === 'biccamera' || $k === 'sofmap' || $k === 'sofmapused' || $k === 'naturum') && !empty($querytype) ) {
					/* biccamera, sofmap, sofmapused and naturum options are keyword only */
					continue;
				}
				$retplus = wfRenderYJShoppingPlusLS( $k, $wgYJShoppingPlus[$k]['lsid'], (!empty($wgYJShoppingPlus[$k]['bids']) ? $wgYJShoppingPlus[$k]['bids'] : ''), $value );
			} elseif( !empty($wgYJShoppingPlus[$k]['a8mat']) ) {
				/* plus A8.net (option) */
				if( ($k === 'nttxstore' || $k === 'hikaritv') && !empty($querytype) ) {
					/* nttxstore and hikaritv options are keyword only */
					continue;
				}
				$retplus = wfRenderYJShoppingPlusA8( $k, $wgYJShoppingPlus[$k]['a8mat'], $value );
			}

			if( !empty($retplus) ) {
				$ret .= '<li ';
				if( !empty($wgYJShoppingClass[$k]) ) {
					$ret .= 'class="' . $wgYJShoppingClass[$k] . '"';
				}
				$ret .= '>' . $retplus . '</li>';
			}
		}

		/* plus kakakucom (option) */
		if( empty($querytype) && !empty($wgYJShoppingPlus['kakakucom']) ) {
			$caption = wfMessage( 'yjshopping-plus-kakakucom' )->inContentLanguage()->escaped();
			$ret .= '<li class="' . $wgYJShoppingClass['kakakucom'] . '"><a href="http://kakaku.com/search_results/' . urlencode($value) . '/" title="' . wfMessage( 'yjshopping-plus-title', $value, $caption )->inContentLanguage()->escaped() . '"' . ($wgYJShoppingTarget ? ' target="' . $wgYJShoppingTarget . '"' : '') . ' rel="nofollow">' . $caption . '</a></li>';
		}

		$ret .= '</ul>';
	}

	/* api credit : https://developer.yahoo.co.jp/attribution/ */
	$ret .= '<a href="' . YJSHOPPING_CREDIT_TARGET . '"' . ($wgYJShoppingTarget ? ' target="' . $wgYJShoppingTarget . '"' : '') . '><img src="' . YJSHOPPING_CREDIT_URL . '" border="0" width="' . YJSHOPPING_CREDIT_SIZE_X . '" height="' . YJSHOPPING_CREDIT_SIZE_Y . '" alt="' . YJSHOPPING_CREDIT_ALT . '" style="position: absolute; left: 0; bottom: 0; margin: 15px 5px 4px 4px;" /></a>';

	$ret .= "</div>\n";

	/* for debug */	defined( 'YJSHOPPING_DEBUG' ) && ( wfDebugLog( YJSHOPPING_DEBUG, "wfRenderYJShopping() complete: \n$ret\n" ) );

	return array( $ret, "markerType" => 'nowiki' );
//	return $ret;	/* MediaWikiを通した出力が中途半端にescapeされる */
//	return Html::rawElement( 'div', [ 'class' => YJSHOPPING_CLASS_DIV ], $ret );	/* 〃 */
#	return array( Html::rawElement( 'div', [ 'class' => YJSHOPPING_CLASS_DIV . ' ' . $wgYJShoppingClass['div'] ], $ret ), "markerType" => 'nowiki' );
}


/* for generate Value-Commerce link
 *
 */
function wfRenderYJShoppingPlusVC( $target, $sid, $pid, $keyword ) {
	global  $wgYJShoppingTarget;

	/* incomplete parameters */
	if( empty($target) || empty($sid) || empty($pid) || empty($keyword) ) {
		return '';
	}

	$target = mb_strtolower($target);

	/* backward compatibility */
	if( strcmp($target, 'yj') === 0 ) {
		$target = 'yahoo';
	}

	/* undefined message */
	if( empty( wfMessage( 'yjshopping-plus-' . $target )->inContentLanguage() ) ) {
		/* for debug */	defined( 'YJSHOPPING_DEBUG' ) && ( wfDebugLog( YJSHOPPING_DEBUG, "wfRenderYJShoppingPlusVC(): yjshopping-plus-$target not defined.\n" ) );
		return '';
	}
	if( empty( wfMessage( 'yjshopping-plus-vc_url-' . $target )->inContentLanguage() ) ) {
		/* for debug */	defined( 'YJSHOPPING_DEBUG' ) && ( wfDebugLog( YJSHOPPING_DEBUG, "wfRenderYJShoppingPlusVC(): yjshopping-plus-vc_url-$target not defined.\n" ) );
		return '';
	}

	/* generate link */
	$caption = wfMessage( 'yjshopping-plus-' . $target )->inContentLanguage()->escaped();
	$vcurl = wfMessage( 'yjshopping-plus-vc_url-' . $target )->inContentLanguage()->rawParams( rawurlencode($keyword) )->plain();
	/* for debug */	defined( 'YJSHOPPING_DEBUG' ) && ( wfDebugLog( YJSHOPPING_DEBUG, "wfRenderYJShoppingPlusVC(): target, caption, vcurl:  $target, $caption, $vcurl\n" ) );

	return '<a href="https://ck.jp.ap.valuecommerce.com/servlet/referral?sid=' . $sid . '&pid=' . $pid . '&vc_url=' . rawurlencode($vcurl) . '" title="' . wfMessage( 'yjshopping-plus-title', $keyword, $caption )->inContentLanguage()->escaped() . '"' . ($wgYJShoppingTarget ? ' target="' . $wgYJShoppingTarget . '"' : '') . ' rel="nofollow">' . $caption . '<img src="https://ad.jp.ap.valuecommerce.com/servlet/gifbanner?sid=' . $sid . '&pid=' . $pid . '" height="1" width="1" border="0" /></a>';
}


/* for generate LinkShare link
 *
 */
function wfRenderYJShoppingPlusLS( $target, $id, $bids, $keyword ) {
	global  $wgYJShoppingTarget;

	/* incomplete parameters */
	if( empty($target) || empty($id) || empty($keyword) ) {
		return '';
	}

	$target = mb_strtolower($target);

	/* undefined message */
	if( empty( wfMessage( 'yjshopping-plus-' . $target )->inContentLanguage() ) ) {
		/* for debug */	defined( 'YJSHOPPING_DEBUG' ) && ( wfDebugLog( YJSHOPPING_DEBUG, "wfRenderYJShoppingPlusLS(): yjshopping-plus-$target not defined.\n" ) );
		return '';
	}
	if( empty( wfMessage( 'yjshopping-plus-ls_murl-' . $target )->inContentLanguage() ) ) {
		/* for debug */	defined( 'YJSHOPPING_DEBUG' ) && ( wfDebugLog( YJSHOPPING_DEBUG, "wfRenderYJShoppingPlusLS(): yjshopping-plus-ls_murl-$target not defined.\n" ) );
		return '';
	}

	/* Biccamera group and Wowma! specified: keyword(s) accepted only Shift-JIS and encoded */
	if( $target == 'biccamera' || $target == 'sofmap'  || $target == 'sofmapused' || $target == 'wowma') {
		$keyword_encoded = rawurlencode(mb_convert_encoding($keyword, 'SJIS'));
	} else {
		$keyword_encoded = rawurlencode($keyword);
	}

	/* generate link */
	$caption = wfMessage( 'yjshopping-plus-' . $target )->inContentLanguage()->escaped();
	$murl = wfMessage( 'yjshopping-plus-ls_murl-' . $target )->inContentLanguage()->rawParams( $id, $keyword_encoded )->plain();
	/* for debug */	defined( 'YJSHOPPING_DEBUG' ) && ( wfDebugLog( YJSHOPPING_DEBUG, "wfRenderYJShoppingPlusLS(): id, target, caption, murl: $id, $target, $caption, $murl\n" ) );

	$ret = '<a href="' . $murl . '" title="' . wfMessage( 'yjshopping-plus-title', $keyword, $caption )->inContentLanguage()->escaped() . '"' . ($wgYJShoppingTarget ? ' target="' . $wgYJShoppingTarget . '"' : '') . ' rel="nofollow">' . $caption . '</a>';
	if( !empty($bids) ) {
		$ret .= '<img width="1" height="1" border="0" src="https://ad.linksynergy.com/fs-bin/show?id=U4SUyajTXJo&bids=' . $bids . '&type=10" />';
	}
	return $ret;
}


/* for generate A8.net link
 *
 */
function wfRenderYJShoppingPlusA8( $target, $id, $keyword ) {
	global  $wgYJShoppingTarget;

	/* incomplete parameters */
	if( empty($target) || empty($id) || empty($keyword) ) {
		return '';
	}

	$target = mb_strtolower($target);

	/* undefined message */
	if( empty( wfMessage( 'yjshopping-plus-' . $target )->inContentLanguage() ) ) {
		/* for debug */	defined( 'YJSHOPPING_DEBUG' ) && ( wfDebugLog( YJSHOPPING_DEBUG, "wfRenderYJShoppingPlusA8(): yjshopping-plus-$target not defined.\n" ) );
		return '';
	}
	if( empty( wfMessage( 'yjshopping-plus-a8_url-' . $target )->inContentLanguage() ) ) {
		/* for debug */	defined( 'YJSHOPPING_DEBUG' ) && ( wfDebugLog( YJSHOPPING_DEBUG, "wfRenderYJShoppingPlusA8(): yjshopping-plus-ls_murl-$target not defined.\n" ) );
		return '';
	}

	/* NTT-X Store specified: keyword(s) accepted only Shift-JIS and encoded */
	if( $target == 'nttxstore') {
		$keyword = rawurlencode(mb_convert_encoding($keyword, 'SJIS'));
	}

	/* generate link */
	$caption = wfMessage( 'yjshopping-plus-' . $target )->inContentLanguage()->escaped();
	$murl = wfMessage( 'yjshopping-plus-a8_url-' . $target )->inContentLanguage()->rawParams( $id, rawurlencode($keyword) )->plain();
	/* for debug */	defined( 'YJSHOPPING_DEBUG' ) && ( wfDebugLog( YJSHOPPING_DEBUG, "wfRenderYJShoppingPlusA8(): id, target, caption, murl: $id, $target, $caption, $murl\n" ) );

	return '<a href="' . $murl . '" title="' . wfMessage( 'yjshopping-plus-title', $keyword, $caption )->inContentLanguage()->escaped() . '"' . ($wgYJShoppingTarget ? ' target="' . $wgYJShoppingTarget . '"' : '') . ' rel="nofollow">' . $caption . '</a>';
}


/* for generate Amazon Associate link
 *
 */
function wfRenderYJShoppingPlusAmazon( $atag, $keyword ) {
	global  $wgYJShoppingTarget;

	/* incomplete parameters */
	if( empty($atag) || empty($keyword) ) {
		return '';
	}
	/* undefined message */
	if( empty( $caption = wfMessage( 'yjshopping-plus-amazon' )->inContentLanguage()->plain() ) ) {
		return '';
	}

	return '<a href="https://www.amazon.co.jp/gp/search?keywords=' . rawurlencode($keyword) . '&__mk_ja_JP=%E3%82%AB%E3%82%BF%E3%82%AB%E3%83%8A&tag=' . $atag . '" title="' . wfMessage( 'yjshopping-plus-title', $keyword, $caption )->inContentLanguage()->escaped() . '"' . ($wgYJShoppingTarget ? ' target="' . $wgYJShoppingTarget . '"' : '') . ' rel="nofollow">' . $caption . '</a>';
}


/* for generate Rakuten Affiliate link
 *
 */
function wfRenderYJShoppingPlusRakuten( $aid, $keyword ) {
	global  $wgYJShoppingTarget;

	/* incomplete parameters */
	if( empty($aid) || empty($keyword) ) {
		return '';
	}
	/* undefined message */
	if( empty( $caption = wfMessage( 'yjshopping-plus-rakuten' )->inContentLanguage()->plain() ) ) {
		return '';
	}

	return '<a href="https://hb.afl.rakuten.co.jp/hgc/' . rawurlencode($aid) . '/?pc=https%3A%2F%2Fsearch.rakuten.co.jp%2Fsearch%2Fmall%2F' . rawurlencode($keyword) . '%2F-%2Ff.1-p.1-s.1-sf.0-st.A-v.2%3Fx%3D0&scid=af_ich_link_urltxt" title="' . wfMessage( 'yjshopping-plus-title', $keyword, $caption )->inContentLanguage()->escaped() . '"' . ($wgYJShoppingTarget ? ' target="' . $wgYJShoppingTarget . '"' : '') . ' rel="nofollow">' . $caption . '</a>';
}


/* for generate Rakuten LinkShare Affiliate link
 *
 */
function wfRenderYJShoppingPlusRakutenLS( $id, $keyword ) {
	global  $wgYJShoppingTarget;

	/* incomplete parameters */
	if( empty($id) || empty($keyword) ) {
		return '';
	}
	/* undefined message */
	if( empty( $caption = wfMessage( 'yjshopping-plus-rakuten' )->inContentLanguage()->plain() ) ) {
		return '';
	}

	return '<a href="https://click.linksynergy.com/fs-bin/click?id=' . rawurlencode($id) . '&offerid=671770&RD_PARM1=https%253A%252F%252Fsearch.rakuten.co.jp%252Fsearch%252Fmall%252F' . rawurlencode($keyword) . '--hybrid--https%253A%252F%252Fsearch.rakuten.co.jp%252Fsearch%252Fmall%252F' . rawurlencode($keyword) . '&subid=&type=10&tmpid=11045" title="' . wfMessage( 'yjshopping-plus-title', $keyword, $caption )->inContentLanguage()->escaped() . '"' . ($wgYJShoppingTarget ? ' target="' . $wgYJShoppingTarget . '"' : '') . ' rel="nofollow">' . $caption . '</a>';
}

?>
